#================
# FILE          : kiwiExtBuilderBase.pm
#----------------
# PROJECT       : openSUSE Build-Service
# COPYRIGHT     : (c) 2013 SUSE LLC
#               :
# AUTHOR        : Robert Schweikert <rjschwei@suse.com>
#               :
# BELONGS TO    : Operating System images
#               :
# DESCRIPTION   : Unit test implementation for the KIWIExtBuilderBase
#               : module
# STATUS        : Development
#----------------
package Test::kiwiExtBuilderBase;

use strict;
use warnings;
use Readonly;

use Common::ktLog;
use Common::ktTestCase;
use base qw /Common::ktTestCase/;

use KIWICommandLine;
use KIWIExtBuilderBase;
use KIWIFilesystemOptions;
use KIWIGlobals;
use KIWIImage;
use KIWIOverlay;
use KIWIXML;

Readonly my $MINBASESIZE => 104_858_624;
#==========================================
# Constructor
#------------------------------------------
sub new {
	# ...
	# Construct new test case
	# ---
	my $this = shift -> SUPER::new(@_);
	$this -> {dataDir} = $this -> getDataDir() . '/kiwiConfigWriterFactory';
	$this -> removeTestTmpDir();
	return $this;
}

#==========================================
# test_ctor
#------------------------------------------
sub test_ctor {
	# ...
	# Test the KIWIExtBuilderBase constructor
	# ---
	my $this = shift;
	my $kiwi = $this -> {kiwi};
	my $confDir = $this -> {dataDir};
	my $cmdL = $this -> __getCommandLineObj();
	my $xml = $this -> __getXMLObj($confDir, $cmdL);
	my $tmpDir = $this -> createTestTmpDir();
	my $image = $this -> __getImageObj($cmdL, $tmpDir, $xml);
	$cmdL -> setImageTargetDir($tmpDir);
	my $builder = KIWIExtBuilderBase -> new($xml, $cmdL, $image);
	my $msg = $kiwi -> getMessage();
	$this -> assert_str_equals('No messages set', $msg);
	my $msgT = $kiwi -> getMessageType();
	$this -> assert_str_equals('none', $msgT);
	my $state = $kiwi -> getState();
	$this -> assert_str_equals('No state set', $state);
	# Test this condition last to get potential error messages
	$this -> assert_not_null($builder);
	$this -> removeTestTmpDir();
	return;
}

#==========================================
# test_ctor_invalidArg1
#------------------------------------------
sub test_ctor_invalidArg1 {
	# ...
	# Test the KIWIExtBuilderBase with invalid first argument
	# ---
	my $this = shift;
	my $kiwi = $this -> {kiwi};
	my $builder = KIWIExtBuilderBase -> new('foo');
	my $msg = $kiwi -> getMessage();
	my $expected = 'KIWIExtBuilderBase: expecting KIWIXML object as '
		. 'first argument.';
	$this -> assert_str_equals($expected, $msg);
	my $msgT = $kiwi -> getMessageType();
	$this -> assert_str_equals('error', $msgT);
	my $state = $kiwi -> getState();
	$this -> assert_str_equals('failed', $state);
	# Test this condition last to get potential error messages
	$this -> assert_null($builder);
	return;
}

#==========================================
# test_ctor_invalidArg2
#------------------------------------------
sub test_ctor_invalidArg2 {
	# ...
	# Test the KIWIExtBuilderBase with invalid second argument
	# ---
	my $this = shift;
	my $kiwi = $this -> {kiwi};
	my $confDir = $this -> {dataDir};
	my $cmdL = $this -> __getCommandLineObj();
	my $xml = $this -> __getXMLObj($confDir, $cmdL);
	my $builder = KIWIExtBuilderBase -> new($xml, 'foo');
	my $msg = $kiwi -> getMessage();
	my $expected = 'KIWIExtBuilderBase: expecting KIWICommandLine '
        . 'object as second argument.';
	$this -> assert_str_equals($expected, $msg);
	my $msgT = $kiwi -> getMessageType();
	$this -> assert_str_equals('error', $msgT);
	my $state = $kiwi -> getState();
	$this -> assert_str_equals('failed', $state);
	# Test this condition last to get potential error messages
	$this -> assert_null($builder);
	return;
}

#==========================================
# test_ctor_invalidArg3
#------------------------------------------
sub test_ctor_invalidArg3 {
	# ...
	# Test the KIWIExtBuilderBase with invalid thried argument
	# ---
	my $this = shift;
	my $kiwi = $this -> {kiwi};
	my $confDir = $this -> {dataDir};
	my $cmdL = $this -> __getCommandLineObj();
	my $xml = $this -> __getXMLObj($confDir, $cmdL);
	my $builder = KIWIExtBuilderBase -> new($xml, , $cmdL, 'foo');
	my $msg = $kiwi -> getMessage();
	my $expected = 'KIWIExtBuilderBase: expecting KIWIImage object '
		. 'as third argument.';
	$this -> assert_str_equals($expected, $msg);
	my $msgT = $kiwi -> getMessageType();
	$this -> assert_str_equals('error', $msgT);
	my $state = $kiwi -> getState();
	$this -> assert_str_equals('failed', $state);
	# Test this condition last to get potential error messages
	$this -> assert_null($builder);
	return;
}

#==========================================
# test_ctor_noArg1
#------------------------------------------
sub test_ctor_noArg1 {
	# ...
	# Test the KIWIExtBuilderBase with no argument
	# ---
	my $this = shift;
	my $kiwi = $this -> {kiwi};
	my $builder = KIWIExtBuilderBase -> new();
	my $msg = $kiwi -> getMessage();
	my $expected = 'KIWIExtBuilderBase: expecting KIWIXML object as '
		. 'first argument.';
	$this -> assert_str_equals($expected, $msg);
	my $msgT = $kiwi -> getMessageType();
	$this -> assert_str_equals('error', $msgT);
	my $state = $kiwi -> getState();
	$this -> assert_str_equals('failed', $state);
	# Test this condition last to get potential error messages
	$this -> assert_null($builder);
	return;
}

#==========================================
# test_ctor_noArg2
#------------------------------------------
sub test_ctor_noArg2 {
	# ...
	# Test the KIWIExtBuilderBase with no second argument
	# ---
	my $this = shift;
	my $kiwi = $this -> {kiwi};
	my $confDir = $this -> {dataDir};
	my $cmdL = $this -> __getCommandLineObj();
	my $xml = $this -> __getXMLObj($confDir, $cmdL);
	my $builder = KIWIExtBuilderBase -> new($xml);
	my $msg = $kiwi -> getMessage();
	my $expected = 'KIWIExtBuilderBase: expecting KIWICommandLine '
        . 'object as second argument.';
	$this -> assert_str_equals($expected, $msg);
	my $msgT = $kiwi -> getMessageType();
	$this -> assert_str_equals('error', $msgT);
	my $state = $kiwi -> getState();
	$this -> assert_str_equals('failed', $state);
	# Test this condition last to get potential error messages
	$this -> assert_null($builder);
	return;
}

#==========================================
# test_ctor_noArg3
#------------------------------------------
sub test_ctor_noArg3 {
	# ...
	# Test the KIWIExtBuilderBase with no third argument
	# ---
	my $this = shift;
	my $kiwi = $this -> {kiwi};
	my $confDir = $this -> {dataDir};
	my $cmdL = $this -> __getCommandLineObj();
	my $xml = $this -> __getXMLObj($confDir, $cmdL);
	my $builder = KIWIExtBuilderBase -> new($xml, $cmdL);
	my $msg = $kiwi -> getMessage();
	my $expected = 'KIWIExtBuilderBase: expecting KIWIImage object '
		. 'as third argument.';
	$this -> assert_str_equals($expected, $msg);
	my $msgT = $kiwi -> getMessageType();
	$this -> assert_str_equals('error', $msgT);
	my $state = $kiwi -> getState();
	$this -> assert_str_equals('failed', $state);
	# Test this condition last to get potential error messages
	$this -> assert_null($builder);
	return;
}

#==========================================
# test_p_calculateMinImageSize
#------------------------------------------
sub test_p_calculateMinImageSize {
    # ...
    # Test the p_calculateMinImageSize method
    # ---
    my $this = shift;
    my $kiwi = $this->{kiwi};
	my $confDir = $this -> {dataDir};
	my $cmdL = $this -> __getCommandLineObj();
	my $xml = $this -> __getXMLObj($confDir, $cmdL);
	my $tmpDir = $this -> createTestTmpDir();
	$cmdL -> setImageTargetDir($tmpDir);
    my $status = open my $WINE, '>', $tmpDir . '/wine';
	$this -> assert_not_null($status);
	print $WINE 'If you do not like the food, drink more wine';
	$status = close $WINE;
	$this -> assert_not_null($status);
    my %init;
    my $fsOpts = KIWIFilesystemOptions -> new(\%init);
    my $res = $cmdL -> setFilesystemOptions ($fsOpts);
    $this -> assert_not_null($res);
	my $image = $this -> __getImageObj($cmdL, $tmpDir, $xml);
    my $builder = KIWIExtBuilderBase -> new($xml, $cmdL, $image);
    my $minSz = $builder -> p_calculateMinImageSize();
    my $msg = $kiwi -> getMessage();
	my $expected = "Calculated mini fs size: 104858624 Bytes\n";
	$this -> assert_str_equals($expected, $msg);
	my $msgT = $kiwi -> getMessageType();
	$this -> assert_str_equals('info', $msgT);
	my $state = $kiwi -> getState();
	$this -> assert_str_equals('No state set', $state);
    $this -> assert_equals($MINBASESIZE, $minSz);
    $this -> removeTestTmpDir();
    return;
}

#==========================================
# Private helper methods
#------------------------------------------
#==========================================
# __getCommandLineObj
#------------------------------------------
sub __getCommandLineObj {
	# ...
	# Return an empty command line
	# ---
	my $cmdL = KIWICommandLine -> new();
	return $cmdL;
}

#==========================================
# __getImageObj
#------------------------------------------
sub __getImageObj {
	# ...
	# Return a basic KIWIImage object
	# ---
	my $this = shift;
	my $cmdL = shift;
	my $imgTree = shift;
	my $xml  = shift;
	my $image = KIWIImage -> new($xml, $imgTree, $imgTree, undef, '/tmp',
								undef, undef, $cmdL);
	return $image;
}

#==========================================
# __getXMLObj
#------------------------------------------
sub __getXMLObj {
	# ...
	# Create an XML object with the given config dir
	# ---
	my $this      = shift;
	my $configDir = shift;
	my $cmdL = shift;
	my $kiwi = $this->{kiwi};
	# TODO
	# Fix the creation of the XML object once the ctor arguments change
	my $xml = KIWIXML -> new(
		$configDir, undef, undef, $cmdL
	);
	if (! $xml) {
		my $errMsg = $kiwi -> getMessage();
		print "XML create msg: $errMsg\n";
		my $msg = 'Failed to create XML obj, most likely improper config '
		. 'path: '
		. $configDir;
		$this -> assert_equals(1, $msg);
	}
	return $xml;
}

1;
